import persistent
from persistent.dict import PersistentDict
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.app import container
from zope.app.component import site, hooks
from zope.interface import implements
from zope.app.authentication import authentication
from zope.app.security.interfaces import IAuthentication
from zope.app.authentication import session
from zope.location.location import LocationProxy
from timeclock import interfaces
from timeclock import person, work

class TimeClockApplication(persistent.Persistent,
                           container.sample.SampleContainer,
                           site.SiteManagerContainer):

    implements(interfaces.ITimeClockApplication,
               IAttributeAnnotatable)

    title = u''

    def __init__(self):
        super(TimeClockApplication, self).__init__()
        self['users'] = person.PersonContainer()
        self['workLogs'] = work.WorkLogContainer()

        # Since we want this to be a site, we need to register a site manager.
        # We want a local site manager here so we will have to create
        # one locally.
        sm = site.LocalSiteManager(self)
        self.setSiteManager(sm)

        # Next we want to set up authentication for our site.
        auth = authentication.PluggableAuthentication()
        auth['credPlugin'] = session.SessionCredentialsPlugin()
        auth.credentialsPlugins = ('credPlugin',)
        auth['authPlugin'] = LocationProxy(self['users'], auth, 'authPlugin')
        auth.authenticatorPlugins = ('authPlugin',)
        sm['default']['PAU'] = auth
        # now to register this PAU with the site manager
        sm.registerUtility(auth, IAuthentication)


    def _newContainerData(self):
        '''to make is persistent?'''
        return PersistentDict()

def getTimeClockApplication(context=None):
    site = hooks.getSite()
    if interfaces.ITimeClockApplication.providedBy(site):
        return site
    else:
        return None
