from zope.app import zapi
from zope.app.container.interfaces import INameChooser
from zope.formlib import form
from zope.lifecycleevent import ObjectModifiedEvent
from zope.app.container.contained import ObjectRemovedEvent
import zope.event
from zope.app.pagetemplate import ViewPageTemplateFile
from zope.security import canAccess

class TimeClockEditForm(form.EditForm):

    _form_finished = False
    
    @form.action("Apply", condition=form.haveInputWidgets)
    def handle_edit_action(self, action, data):
        if form.applyChanges(self.context, self.form_fields, data, self.adapters):
            zope.event.notify(ObjectModifiedEvent(self.context))
            self.status = "Changes Made"
        else:
            self.status = 'No changes'
        self._form_finished = True

    def render(self):
        if self._form_finished:
            self.request.response.redirect(self.nextURL())
            return ""
        return super(TimeClockEditForm, self).render()

    def nextURL(self):
        return zapi.absoluteURL(self.context, self.request)

class ContainerDisplayForm(form.Form):
    """Form for displaying things from a container"""
    template = ViewPageTemplateFile('container.pt')
    form_fields = form.Fields()
    actions = form.Actions()
    canDelete = False


class ContainerEditForm(ContainerDisplayForm):
    """Form for deleting things from a container."""
    canDelete = True
    @form.action("Delete")
    def handle_delete_action(self, action, data):
        for key in self.request:
            if key.startswith('delete.'):
                name = key[7:]
                if name in self.context:
                    ob = self.context[name]
                    del self.context[name]
                    zope.event.notify(ObjectRemovedEvent(ob))
    
    @property
    def canDelete(self):
        return canAccess(self.context, '__del__')
