from zope import schema, interface
from zope.app.securitypolicy.interfaces import IPrincipalRoleManager
from zope.app.securitypolicy.interfaces import IPrincipalPermissionManager
from zope.app.form.browser.add import AddView
from zope.app.form.interfaces import WidgetsError
from zope import component
from zope.app import zapi
from zope.app.container.interfaces import INameChooser
from zope.formlib import form
from zope.lifecycleevent import ObjectCreatedEvent, ObjectModifiedEvent
import zope.event
import pytz

from timeclock.interfaces import IPerson
from timeclock.person import Person
from timeclock.work import WorkLog
from timeclock.app import getTimeClockApplication
from timeclock.browser import app

class PersonAddFormSchema(IPerson):
    """Schema for the Person Add Form"""

    password_confirm = schema.Password(
        title=u"Confirm Password",
        required=True)

    role = schema.Choice(
        title=u"Role",
        values=[u"employee",u"manager"],
        required=True)

    createWorkLog = schema.Bool(
        title=u"Create Work Log for this User",
        required=True)

class PersonAddForm(form.AddForm):

    form_fields = form.Fields(PersonAddFormSchema).select('title','login',
                                                          'password','password_confirm',
                                                          'role', 'createWorkLog')

    label="Add a Person"

    def createAndAdd(self, data):
        if data['password'] != data['password_confirm']:
            self.status = 'There was an error'
            self.errors += ('passwords do not match',)
            return False
        
        person = Person(data['login'],
                        data['password'],
                        data['title'],
                        passwordManagerName="MD5")
        component.provideUtility(person, IPerson, data['login'])
        zope.event.notify(ObjectCreatedEvent(person))

        ppm = IPrincipalPermissionManager(person)
        ppm.grantPermissionToPrincipal('timeclock.edit', person.login)

        self.context[data['login']] = person

        role = data['role']
        if role:
            role = 'timeclock.'+role
            app = getTimeClockApplication()
            prinRole = IPrincipalRoleManager(app)
            prinRole.assignRoleToPrincipal(role, data['login'])

        if data['createWorkLog']:
            worklogs = getTimeClockApplication()['workLogs']
            worklog = WorkLog()
            worklog.person = person
            worklog.title = data['title']+"'s Work Log"
            worklogs[data['login']] = worklog
            ppm = IPrincipalPermissionManager(worklog)
            ppm.grantPermissionToPrincipal('timeclock.edit', person.login)
            zope.event.notify(ObjectCreatedEvent(worklog))

        self._finished_add = True
        return person

    def nextURL(self):
        return zapi.absoluteURL(self.context, self.request)

class PersonDisplayForm(object):

    def workLogs(self):
        worklogs = getTimeClockApplication(None)['workLogs']
        for log in worklogs.values():
            if log.person == self.context:
                yield log

class PersonEditForm(app.TimeClockEditForm):

    form_fields = form.Fields(IPerson).select('title','login','password','timeZone')
    label="Edit a Person"


class PersonContainerForm(app.ContainerEditForm):

    title = u"Persons Container"

class AbsoluteURL(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        return zapi.absoluteURL(getTimeClockApplication(None)['users'],
                                self.request)+'/'+self.request.principal.id
        
