from zope.interface import Interface 
from zope import schema
from zope.app.container.interfaces import IContainer
from zope.app.container.interfaces import IContained
from zope.app.container.interfaces import IReadContainer, IWriteContainer
from zope.app.container.constraints import contains, containers
from zope.app.component.interfaces import IPossibleSite
from zope.app.authentication.principalfolder import IInternalPrincipal
from zope.app.security.vocabulary import PrincipalSource
import pytz

class ITimeClockApplication(IReadContainer, IPossibleSite):
    """This is a TimeClock Application."""
    title = schema.TextLine(title=u"Title",
                     description=u"The Name of this Application",
                     required=True)

class IPerson(IInternalPrincipal):
    """A Person object and principal"""

    timeZone = schema.Choice(
        title=u"Time Zone",
        values=pytz.common_timezones,
        required=False)


class IPersonContainer(IContainer):
    """A container for people"""
    contains(IPerson)


class IPersonContained(IContained):
    """A Person contained in a IPersonContainer"""
    containers(IPersonContainer)


class IWorkLogEntry(Interface):
    """A work entry."""
    startTime = schema.Datetime(
        title=u"Start Time",
        description=u"Person's start time",
        required=True)

    endTime = schema.Datetime(
        title=u"End Time",
        description=u"Person's end time",
        required=False)

    workDone = schema.Text(
        title=u"Work Done",
        description=u"What s/he did",
        required=False)

    learned = schema.Text(
        title=u"What you learned",
        description=u"What s/he learned",
        required=False)

    cost = schema.Float(
        title=u"Cost of this work",
        required=False)

    totalTime = schema.Float(
        title=u"Total time in hours.",
        required=False)


class IReadWorkLog(IReadContainer):
    """A container for work entries."""

    person = schema.Field(
        title=u'Employee',
        description=u'Employee whose work is being tracked',
        required=True)

    title = schema.TextLine(
        title=u"Work Log Title",
        description=u"Title of Work Log",
        required=False)

    clockedIn = schema.Bool(
        title=u"Clocked In",
        description=u"Whether or not the person is clocked in.",
        required=False)

    clockedInTime = schema.Datetime(
        title=u"Time of Last Clock In",
        required=False)

    hourlyPay = schema.Float(
        title=u"Hourly Pay",
        required=False)

    reportHeader = schema.Text(
        title=u"Report Header",
        description=u"The thing that should go at the top of a report.",
        required=False)

    submitted = schema.Field(
        title=u"submitted work logs",
        required=True)

    totalCost = schema.Float(
        title=u"Total Cost",
        description=u"Total cost of work in this work log thus far.",
        required=False)

    totalTime = schema.Float(
        title=u"Total Time",
        description=u"Total time of work in this work log thus far.",
        required=False)

    contains(IWorkLogEntry)

class IWriteWorkLog(IWriteContainer):

    def submitEntries(startDate, endDate):
        """Submit a set of entries in the work log."""

class IWorkLog(IReadWorkLog, IWriteWorkLog):
    """Both reading and writing"""

class ISubmittedWorkLog(IReadWorkLog, IWriteContainer):
    """A worklog that has been submitted."""

    workLog = schema.Field(
        title=u"The Original WorkLog",
        required=True)

    title = schema.TextLine(title=u"Title", required=True)

    status = schema.Choice(
        title=u"Status",
        values=["Newly Submitted",
                "Processed",
                "Payment Recieved"],
        required=True)

    contains(IWorkLogEntry)


class ISubmittedWorkLogContainer(IReadContainer):

    contains(ISubmittedWorkLog)


class IWorkLogEntryContained(IContained):
    """A WorkLogEntry contained in a WorkLogEntryContainer"""
    containers(IWorkLog, ISubmittedWorkLog)


class IWorkLogContainer(IContainer):
    """A container for worklog."""
    contains(IWorkLog)


class IWorkLogContained(IContained):
    """A WorkLog contained in WorkLogContainer."""
    containers(IWorkLogContainer)
